#include <BluetoothSerial.h>
#include "Ticker.h"

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it
#endif

BluetoothSerial SerialBT;
Ticker timerObj;
int time_count=0;
const int cIntvTime = 50; //タイマー割り込み５０ｍｓｅｃ

bool btConnect=false;
bool btRecv=false;
bool comRecv=false;
bool btErr=false;

#define BTCONNECT_LED 18
#define COMRECV_LED 21
#define BTRECV_LED 19

void setup() {
  // put your setup code here, to run once:
  //デバッグシリアルポート設定
  Serial.begin(115200); 
  
  //インジケータ用LEDピンの設定
  pinMode(BTCONNECT_LED, OUTPUT);
  pinMode(BTRECV_LED, OUTPUT);
  pinMode(COMRECV_LED, OUTPUT);

  //BTの名前設定「ＢＴ－ＡＤＰ」
  char btname[11];
  //sprintf(btname,"ESP32-%02X%02X",macBT[4],macBT[5]);
  sprintf(btname,"BT-ADP");
  
  //Bluetoothのcallback関数設定
  SerialBT.register_callback(callbackBT);  
  //Bluetooth開始
  if(SerialBT.begin(btname)){
    btErr=false;
    Serial.println("Blueooth Object Ready!!");
  }else{  //失敗なら・・・
    btErr=true;
    Serial.println("Blueooth Object Error!!");
  }

  //Serial2(RX=GPIO16,TX=GPIO17)を開始（ＣＯＭ側）
  Serial2.begin(4800,SERIAL_8N1); //ICOMは８ビット・パリティ無し・ＳＴＯＰ１ビット
                                  //他は８ビット・パリティ無し・ＳＴＯＰ２ビット
                                  //ＴＣ－ＡＨ＄・ＳＤ－ＣＮＴは４８００ｂｐｓ

  //タイマー割り込み開始
  timerObj.attach_ms(cIntvTime, timer_interrupt);
}

void loop() {
  // put your main code here, to run repeatedly:
  if (Serial2.available()) {  //ＣＯＭ側受信あれば
    comRecv=true;
    SerialBT.write(Serial2.read()); //ＢＴへ転送
  }
  if (SerialBT.available()) { //ＢＴ側受信あれば
    btRecv=true;
    Serial2.write(SerialBT.read()); //ＣＯＭへ転送
  }
}

void callbackBT(esp_spp_cb_event_t event, esp_spp_cb_param_t *param) {
  if (event == ESP_SPP_SRV_OPEN_EVT) {  //ＢＴオープンされたら
    btConnect=true;
  }else if (event == ESP_SPP_CLOSE_EVT) { //ＢＴクローズされたら
    btConnect=false;
  }
}


void timer_interrupt(){
  //時間１秒まで繰り返しカウント
  time_count+=cIntvTime;
  if(time_count>=1000)time_count=0;

  //BTCONNECTのフリッカー動作
  if(btErr==false){
    if(time_count==0){
      digitalWrite(BTCONNECT_LED, HIGH); //とりあえず点灯
    }
    if(time_count==500){
      if(btConnect==false)digitalWrite(BTCONNECT_LED, LOW); //未接続なら点滅のため消灯
    }  
  }else{
    //BT object 失敗なら
    if(digitalRead(BTCONNECT_LED)==HIGH){
      digitalWrite(BTCONNECT_LED, LOW);
    }else{
      digitalWrite(BTCONNECT_LED, HIGH);
    }
  }

  //BTRECVの動作
  if(btRecv==true){ //受信あれば・・
    digitalWrite(BTRECV_LED, HIGH);  //点灯
    btRecv=false;
  }else{            //通常は
    digitalWrite(BTRECV_LED, LOW); //消灯しておく
  }

  //COMRECVの動作
  if(comRecv==true){ //受信あれば・・
    digitalWrite(COMRECV_LED, HIGH);  //点灯
    comRecv=false;
  }else{            //通常は
    digitalWrite(COMRECV_LED, LOW); //消灯しておく
  }
}
